-----------------------------------------------------------
--|> SLASH COMMANDS
-----------------------------------------------------------
--> Namespace
local ADDON_NAME, NS = ...
--> Upvalues
local wipe = wipe
local Reload_UI = C_UI.Reload


--|> ACE3 SLASH COMMANDS TABLE
-----------------------------------------------------------
NS.SlashCommands = {
    name = "Slash Commands",
    order = -3,
    type = "group",
    args = {
        intro = {
            name = "weizPVP Slash Commands",
            type = "description",
            order = 1,
            cmdHidden = true
        },
        config = {
            name = "config",
            desc = "Opens the configuration menu.",
            type = "execute",
            func = function()
                NS.ToggleOptions()
            end,
            guiHidden = true,
            order = 2
        },
        show = {
            name = "Show",
            desc = "Show the weizPVP window.",
            type = "execute",
            order = 3,
            func = function()
                NS.SetWindowVisible(true)
                NS.Options.Window.Visible = true
            end,
            dialogHidden = true
        },
        hide = {
            name = "Hide",
            desc = "Hide the weizPVP window.",
            type = "execute",
            order = 4,
            func = function()
                NS.SetWindowVisible(false)
                NS.Options.Window.Visible = false
            end,
            dialogHidden = true
        },
        lock = {
            name = "Lock",
            desc = "Toggle locking of the window's position and size.",
            type = "execute",
            order = 5,
            func = function()
                NS.SetWindowLocked(not NS.Options.Window.Locked)
            end,
            dialogHidden = true
        },
        pin = {
            name = "Pin",
            desc = "Toggle pinning of the window.",
            type = "execute",
            order = 6,
            func = function()
                if NS.Options.Window.Locked then
                    NS.PrintAddonMessage("Window is currently |cffff3838LOCKED|r. Window can only be pinned after being unlocked.")
                else
                    NS.SetWindowPin(not NS.Options.Window.Pinned)
                end
            end,
            dialogHidden = true
        },
        ch = {
            name = "Crosshair",
            desc = "Toggles the Crosshair on or off",
            type = "execute",
            order = 7,
            func = function()
                if NS.Options.Crosshair.Enabled == false then
                    NS.Options.Crosshair.Enabled = true
                    NS.Crosshair.Enable()
                    NS.PrintAddonMessage("Crosshair is |cff37ff37ENABLED|r.")
                else
                    NS.Options.Crosshair.Enabled = false
                    NS.Crosshair.Disable()
                    NS.PrintAddonMessage("Crosshair is |cffff3838DISABLED|r.")
                end
            end,
            dialogHidden = true
        },
        pb = {
            name = "Player Browser",
            desc = "Toggle the Player Browser",
            type = "execute",
            order = 8,
            func = function()
                if NS.PlayerBrowser:IsShown() then
                    NS.PlayerBrowser:Hide()
                else
                    NS.PlayerBrowser:Show()
                end
            end,
            dialogHidden = true
        },
        debug = {
            name = "DEBUG",
            desc = "Toggles DEBUG mode where ALL players of any factions are accepted as valid.",
            type = "execute",
            order = 60,
            func = function()
                if NS.Options.DEBUG == true then
                    NS.Options.DEBUG = false
                    weizPVP.NS = nil
                    NS.ClearListData()
                    NS.PrintAddonMessage("|TInterface\\Addons\\weizPVP\\Media\\Icons\\toolbox.tga:0|t - |cff37ff37DISABLED|r")
                    NS.SetStatusBarMessage("|cff37ff37> DEBUG MODE DISABLED <|r", true, 3)
                else
                    NS.Options.DEBUG = true
                    weizPVP.NS = NS
                    NS.PrintAddonMessage("|TInterface\\Addons\\weizPVP\\Media\\Icons\\toolbox.tga:0|t - |cffff3838ENABLED|r")
                    NS.SetStatusBarMessage("|cffff3838>|r DEBUG MODE: |cffff3838ACTIVE <|r", true, 3)
                end
            end,
            cmdHidden = true
        },
        --> RESETALL: Wipes EVERYTHING
        resetall = {
            cmdHidden = true,
            name = "Reset All",
            desc = "Resets all settings and wipes player data",
            type = "execute",
            order = 61,
            func = function()
                NS.ResetAll()
            end
        },
        --> RESET OPTIONS
        resetoptions = {
            cmdHidden = true,
            name = "Reset Options",
            desc = "Resets all Options but doesn't change player data at all",
            type = "execute",
            order = 63,
            func = function()
                NS.ResetOptions()
            end
        },
        --> RESET PLAYER DB
        resetPlayers = {
            cmdHidden = true,
            name = "Reset Player DB",
            desc = "Wipes the player database. Options remain untouched.",
            type = "execute",
            order = 64,
            func = function()
                NS.ResetPlayerDB()
            end
        }
    }
}

LibStub("AceConfig-3.0"):RegisterOptionsTable(ADDON_NAME .. " Commands", NS.SlashCommands, {"wpvp", "weizpvp"}) -- Register Slash Commands

--|> STATIC POPUPS
-----------------------------------------------------------
--> Reset All Popup
StaticPopupDialogs["WEIZPVP_CONFIRM_RESET_ALL"] = {
    text = "Are you sure you want to reset all weizPVP settings and wipe its player data?\n|cffff0000This will reload your UI.|r",
    button1 = YES,
    button2 = NO,
    OnAccept = function()
        wipe(NS.charDB)
        wipe(NS.globalDB)
        wipe(NS.Options)
        wipe(NS.PlayerDB)
        wipe(NS.KosList)
        wipe(_weizpvp_chardb)
        wipe(_weizpvp_global_info)
        wipe(_weizpvp_globaldb)
        Reload_UI()
    end,
    timeout = 0,
    whileDead = 1,
    hideOnEscape = 1,
    showAlert = 1,
    OnShow = function(self)
        self:SetFrameStrata("FULLSCREEN_DIALOG")
        self:SetFrameLevel(99)
    end
}
--> Reset Options
StaticPopupDialogs["WEIZPVP_CONFIRM_RESET_OPTIONS"] = {
    text = "Are you sure you want to reset the weizPVP options & settings?\n|cffff0000This will reload your UI.|r",
    button1 = YES,
    button2 = NO,
    OnAccept = function()
        if NS.charDB.profile and NS.charDB.profile.Options then
            NS.charDB.profile.Options = nil
        end
        NS.Options = nil
        Reload_UI()
    end,
    timeout = 0,
    whileDead = 1,
    hideOnEscape = 1,
    showAlert = 1,
    OnShow = function(self)
        self:SetFrameStrata("FULLSCREEN_DIALOG")
        self:SetFrameLevel(99)
    end
}
--> Reset Player Database
StaticPopupDialogs["WEIZPVP_CONFIRM_RESET_PLAYER_DB"] = {
    text = "Are you sure you want to wipe the player database?",
    button1 = YES,
    button2 = NO,
    OnAccept = function()
        wipe(NS.globalDB)
        wipe(NS.PlayerDB)
        wipe(NS.PlayerActiveCache)
        wipe(_weizpvp_globaldb)
        NS.globalDB = LibStub("AceDB-3.0"):New("_weizpvp_globaldb", {}, false)
        NS.PlayerDB = NS.globalDB.global
        NS.ClearListData()
    end,
    timeout = 0,
    whileDead = 1,
    hideOnEscape = 1,
    showAlert = 1,
    OnShow = function(self)
        self:SetFrameStrata("FULLSCREEN_DIALOG")
        self:SetFrameLevel(99)
    end
}
--> Version Update - Options and DB reset
StaticPopupDialogs["WEIZPVP_UPGRADE_DB_RESET"] = {
    text = "|cffFFA200weizPVP options and data have been reset due to some major updates!|r\n|cffaaaaaa(Details in chat)|r",
    button1 = OKAY,
    button2 = nil,
    OnAccept = function()
    end,
    timeout = 0,
    whileDead = 1,
    hideOnEscape = 1,
    showAlert = 1,
    OnShow = function(self)
        self:SetFrameStrata("FULLSCREEN_DIALOG")
        self:SetFrameLevel(99)
    end
}
