-----------------------------------------------------------
--|> Core UI: Utils
-----------------------------------------------------------
local _, NS = ...
local abs = abs

--> SCALE COORDS
--: IN: X,Y coords of where to place something
--: OUT: X, Y coords based effective scale of the game window.fi
function NS.ScaleCoords(xPixel, yPixel)
    local xFactor = NS.Display.xFactor
    local yFactor = NS.Display.yFactor
    local x, y = xPixel / xFactor, yPixel / yFactor
    x, y = x - x % 1, y - y % 1
    return x * xFactor, y * yFactor
end

--> GET FRAME POSITION
function NS.GetFramePosition(frame)
    if not frame then
        return
    elseif frame:IsProtected() then
        return
    elseif not frame:GetLeft() then
        return
    end
    if frame then
        local parent = frame:GetParent() or nil
        local s = frame:GetScale()
        local left, top = frame:GetLeft() * s, frame:GetTop() * s
        local right, bottom = frame:GetRight() * s, frame:GetBottom() * s
        local w, h = frame:GetWidth() * s, frame:GetHeight() * s
        local pWidth, pHeight
        if not parent then
            pWidth = UIParent:GetWidth()
            pHeight = UIParent:GetHeight()
        else
            pWidth, pHeight = parent:GetWidth(), parent:GetHeight()
        end
        local x, y
        local point
        if left < (pWidth - right) and left < abs((left + right) / 2 - pWidth / 2) then
            x = left
            point = "LEFT"
        elseif (pWidth - right) < abs((left + right) / 2 - pWidth / 2) then
            x = right - pWidth
            point = "RIGHT"
        else
            x = (left + right) / 2 - pWidth / 2
        end
        if bottom < (pHeight - top) and bottom < abs((bottom + top) / 2 - pHeight / 2) then
            y = bottom
            if point then
                point = "BOTTOM" .. point
            else
                point = "BOTTOM"
            end
        elseif (pHeight - top) < abs((bottom + top) / 2 - pHeight / 2) then
            y = top - pHeight
            if point then
                point = "TOP" .. point
            else
                point = "TOP"
            end
        else
            y = (bottom + top) / 2 - pHeight / 2
        end
        point = point or "CENTER"
        frame:ClearAllPoints()
        frame:SetPoint(point, UIParent, point, x / s, y / s)
        frame:SetWidth(w / s)
        frame:SetHeight(h / s)
        return x, y, w, h, s, point
    end
    return nil
end

--> Save Core Position
function NS.SaveCoreFramePosition()
    local x, y, w, h, s, point = NS.GetFramePosition(NS.CoreUI.CoreFrame)
    NS.Options.Frames.X = x
    NS.Options.Frames.Y = y
    NS.Options.Frames.Width = w
    NS.Options.Frames.Height = h
    NS.Options.Frames.Point = point
    NS.Options.Frames.Scale = s
    NS.CoreUI.ChangeTargetIcon()
end

--> Set Core Position
function NS.SetCoreFramePosition()
    local settings = {}
    settings.xC = NS.Options.Frames.X
    settings.yC = NS.Options.Frames.Y
    settings.wC = NS.Options.Frames.Width
    settings.hC = NS.Options.Frames.Height
    settings.pointC = NS.Options.Frames.Point or "CENTER"
    settings.sC = NS.Options.Frames.Scale
    settings.As = NS.Options.Frames.AutoScaleEnabled
    NS.CoreUI.CoreFrame:ClearAllPoints()
    NS.CoreUI.CoreFrame:SetHeight(settings.hC / settings.sC)
    NS.CoreUI.CoreFrame:SetWidth(settings.wC / settings.sC)
    if not settings.As then
        NS.CoreUI.CoreFrame:SetScale(settings.sC)
    else
        NS.CoreUI.CoreFrame:SetScale(ConvertPixelsToUI(1, UIParent:GetEffectiveScale()))
        NS.Options.Frames.Scale = ConvertPixelsToUI(1, UIParent:GetEffectiveScale())
    end
    NS.CoreUI.CoreFrame:SetPoint(settings.pointC, UIParent, settings.pointC, settings.xC / settings.sC, settings.yC / settings.sC)
end

--> Format Player Name for Display
local gsub = gsub
local string_find = string.find
local WrapTextInColorCode = WrapTextInColorCode
local GetClassColor = GetClassColor
local select = select
function NS.FormatPlayerNameForDisplay(playerName)
    local charName = WrapTextInColorCode(gsub(playerName, "-(.*)", ""), select(4, GetClassColor(NS.PlayerDB[playerName].Class)))
    if not string_find(playerName, NS.PlayerRealm) then
        charName = charName .. "|cFFFF00CC*|r"
    end
    return charName
end

--> Format Relative Level for Display
-- @tparam number level Incoming level to format and color based on difference in levels vs the current player
function NS.FormatRelativeLevelForDisplay(level)
    if level then
        local playerLevel = NS.Player.Level
        if level == 0 then --> 0
            return "|cFFFF00CC??|r"
        elseif level < playerLevel - 20 then --> 20+ below
            return "|cFF7cffd1" .. level .. "|r"
        elseif level < playerLevel - 10 then --> 10-20 below
            return "|cFF7cd1ff" .. level .. "|r"
        elseif level < playerLevel then --> 1-9 below
            return "|cFF7cff7f" .. level .. "|r"
        elseif level > playerLevel then --> Higher level
            return "|cFFf7694a" .. level .. "|r"
        else
            return "|cFFffc863" .. level .. "|r"
        end
    end
end
