-----------------------------------------------------------------------
--|> EVENTS
-----------------------------------------------------------------------
local _, NS = ...
local string_find = string.find
local UnitIsDeadOrGhost = UnitIsDeadOrGhost

NS.PlayerActiveCache = {}

--|> EVENT MONITORING
-----------------------------------------------------------------------
--> ENABLE EVENTS
function NS.EnableEvents()
    --> ADDON
    weizPVP:RegisterEvent("ADDON_LOADED", NS.AddonLoadedEvent)
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", NS.NameplateAdded)
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", NS.NameplateRemoved)
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", NS.CombatLogEvent)
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", NS.PlayerTargetEvent)
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", NS.PlayerMouseoverEvent)
    weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", NS.UnitHealthEvent)
    weizPVP:RegisterEvent("UNIT_TARGET", NS.UnitTargetEvent)
    --> ZONE CHANGED
    weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", NS.PlayerEnteringBattlegroundEvent)
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", NS.PlayerEnteringWorldEvent)
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", NS.ZoneChangedNewAreaEvent)
    --> COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", NS.EnteringCombat)
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", NS.LeavingCombat)
    --> PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", NS.PlayerLevelUp)
    --> DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", NS.UpdateDisplayData)
    weizPVP:RegisterEvent("PLAYER_LOGIN", NS.UpdateDisplayData)
    --> Crosshair
    NS.Crosshair.Enable()
end

--> DISABLE EVENTS
function NS.DisableEvents()
    --> ADDON
    weizPVP:UnregisterEvent("ADDON_LOADED", NS.AddonLoadedEvent)
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", NS.NameplateAdded)
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", NS.NameplateRemoved)
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", NS.CombatLogEvent)
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", NS.PlayerTargetEvent)
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", NS.PlayerMouseoverEvent)
    weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", NS.UnitHealthEvent)
    weizPVP:UnregisterEvent("UNIT_TARGET", NS.UnitTargetEvent)
    --> COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", NS.EnteringCombat)
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", NS.LeavingCombat)
    --> PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", NS.PlayerLevelUp)
    --> DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", NS.UpdateDisplayData)
    weizPVP:UnregisterEvent("PLAYER_LOGIN", NS.UpdateDisplayData)
    -->Crosshair
    NS.Crosshair.Disable()
end

--|> UNIT EVENTS
-----------------------------------------------------------------------
--> EVENT: MOUSEOVER
function NS.PlayerMouseoverEvent()
    NS.GetUnitData("mouseover")
end

--> EVENT: TARGET CHANGED
function NS.PlayerTargetEvent()
    NS.GetUnitData("target")
    NS.Crosshair.NewTarget() -- Crosshair-New Target
    NS.CoreUI.ChangeTargetIcon()
    NS.UnitHealthEvent(_, "target") --extra health check
end

--> EVENT: NAMEPLATE ADDED
function NS.NameplateAdded(_, unit)
    -- API bug workaround
    if not unit then
        return
    end
    if string_find(NS.GetUnitName(unit), "Unknown") then
        return false
    end
    NS.Crosshair.NameplateAdded(unit)
    NS.GetUnitData(unit)
    NS.CurrentNameplates[UnitGUID(unit)] = true
end

--> EVENT: NAMEPLATE REMOVED
function NS.NameplateRemoved(_, unit)
    -- API bug workaround
    if not unit then
        return
    end
    if string_find(NS.GetUnitName(unit), "Unknown") then
        return false
    end
    NS.Crosshair.NameplateRemoved(unit)
    NS.GetUnitData(unit)
    if NS.IsUnitValidForTracking(unit) then
        NS.CurrentNameplates[UnitGUID(unit)] = nil
    end
end

--> EVENT: HEALTH
function NS.UnitHealthEvent(_, unit)
    if NS.IsUnitValidForTracking(unit) then
        local GUID = UnitGUID(unit) or nil
        -- Check if player is already on bars
        if GUID and NS.PlayersOnBars[GUID] and NS.PlayerActiveCache[GUID] then
            local currentHealth = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
            local dead = UnitIsDeadOrGhost(unit)
            -- If valid dead value that is different that previously stored; update.
            if dead ~= nil and dead ~= NS.PlayerActiveCache[GUID].Dead then
                NS.PlayerActiveCache[GUID].Dead = dead
            end
            if currentHealth ~= nil and currentHealth ~= NS.PlayerActiveCache[GUID].Health then
                NS.PlayerActiveCache[GUID].Health = currentHealth
            end
        end
    end
end

--> EVENT: UNIT TARGET
function NS.UnitTargetEvent(_, unit)
    NS.GetUnitData(unit)
end

--|> ADDON EVENTS
-----------------------------------------------------------------------
--> EVENT: ADDON_LOADED
function NS.AddonLoadedEvent()
    NS.Crosshair.OnLoad()
    weizPVP:UnregisterEvent("ADDON_LOADED", "AddonLoadedEvent") --disabled after initial load
end

--|> COMBAT EVENTS
-----------------------------------------------------------------------
--> EVENT: ENTERING COMBAT
function NS.EnteringCombat()
    NS.SetStatusBarMessage("|cFFF4564D+++ IN COMBAT +++|r", false)
end

--> EVENT: LEAVING COMBAT
function NS.LeavingCombat()
    NS.SetStatusBarMessage("|cFF53F493--- OUT OF COMBAT ---|r", true, 3)
    NS.ManageListTimeouts()
end
