local XEN = XEN
XEN.SnapUtil = {}

local SnapUtil = XEN.SnapUtil

--> Get Pixel To UI Unit Factor
function XEN.SnapUtil.GetPixelToUIUnitFactor()
    local physicalWidth, physicalHeight = GetPhysicalScreenSize()
    return 768.0 / physicalHeight
end

--> Get Nearest Pixel Size
function XEN.SnapUtil.GetNearestPixelSize(uiUnitSize, layoutScale, minPixels)
    if uiUnitSize == 0 and (not minPixels or minPixels == 0) then
        return 0
    end
    local uiUnitFactor = SnapUtil.GetPixelToUIUnitFactor()
    local numPixels = Round((uiUnitSize * layoutScale) / uiUnitFactor)
    local rawNumPixels = numPixels
    if minPixels then
        if uiUnitSize < 0.0 then
            if numPixels > -minPixels then
                numPixels = -minPixels
            end
        else
            if numPixels < minPixels then
                numPixels = minPixels
            end
        end
    end
    return numPixels * uiUnitFactor / layoutScale
end

--> Set Width
function XEN.SnapUtil.SetWidth(region, width, minPixels)
    region:SetWidth(SnapUtil.GetNearestPixelSize(width, region:GetEffectiveScale(), minPixels))
end

--> SetHeight
function XEN.SnapUtil.SetHeight(region, height, minPixels)
    region:SetHeight(SnapUtil.GetNearestPixelSize(height, region:GetEffectiveScale(), minPixels))
end

function XEN.SnapUtil.SetSize(region, width, height, minWidthPixels, minHeightPixels)
    SnapUtil.SetWidth(region, width, minWidthPixels)
    SnapUtil.SetHeight(region, height, minHeightPixels)
end

function XEN.SnapUtil.SetPoint(
    region,
    point,
    relativeTo,
    relativePoint,
    offsetX,
    offsetY,
    minOffsetXPixels,
    minOffsetYPixels)
    region:SetPoint(
        point,
        relativeTo,
        relativePoint,
        SnapUtil.GetNearestPixelSize(offsetX, region:GetEffectiveScale(), minOffsetXPixels),
        SnapUtil.GetNearestPixelSize(offsetY, region:GetEffectiveScale(), minOffsetYPixels)
    )
end

function XEN.SnapUtil.SetStatusBarValue(statusBar, value)
    local width = statusBar:GetWidth()
    if width and width > 0.0 then
        local min, max = statusBar:GetMinMaxValues()
        local percent = ClampedPercentageBetween(value, min, max)
        if percent == 0.0 or percent == 1.0 then
            statusBar:SetValue(value)
        else
            local numPixels = SnapUtil.GetNearestPixelSize(statusBar:GetWidth() * percent, statusBar:GetEffectiveScale())
            local roundedValue = Lerp(min, max, numPixels / width)
            statusBar:SetValue(roundedValue)
        end
    else
        statusBar:SetValue(value)
    end
end
