-----------------------------------------------------------
--|> UNIT PROCESSING
--: Process Unit Data
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local string_find = string.find
local UnitCanAttack = UnitCanAttack
local UnitExists = UnitExists
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitOnTaxi = UnitOnTaxi
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local gsub = gsub

--> GET UNIT DATA
-- Gets data for events with a UNIT ID; such as 'target' or 'nameplate'
function NS.GetUnitData(unit)
    if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
        return -- no valid unit to process
    end
    -- valid unit exists: get info!
    local name = NS.GetUnitName(unit)
    local GUID = UnitGUID(unit) or nil
    if name and GUID then
        local timestamp = GetTime()
        local guild = GetGuildInfo(unit)
        local class
        _, class = UnitClass(unit)
        local role = NS.ClassRoleAssign(UnitClass(unit))
        local level = UnitLevel(unit)
        local race = UnitRace(unit)
        local OnTaxi = UnitOnTaxi(unit)
        -- Add player to DB if not found
        if not NS.PlayerDB[name] or NS.PlayerDB[name].Estimated == true then
            NS.PlayerDB[name] = {}
            NS.PlayerDB[name].Name = name
            NS.PlayerDB[name].Guild = guild
            NS.PlayerDB[name].Class = class
            if role ~= nil then
                NS.PlayerDB[name].Role = role
            end
            NS.PlayerDB[name].Level = level
            NS.PlayerDB[name].Race = race
        end
        -- get more info if player is on bars
        if NS.PlayersOnBars[GUID] and NS.PlayerActiveCache[GUID] then
            if OnTaxi then
                NS.PlayerActiveCache[GUID].OnTaxi = true
            elseif not OnTaxi and NS.PlayerActiveCache[GUID].OnTaxi == true then
                NS.PlayerActiveCache[GUID].OnTaxi = false
            end
            NS.UnitHealthCheck(unit)
        end
        NS.PlayerDB[name].Estimated = false
        NS.PlayerDB[name].Timestamp = timestamp
        NS.UpdatePlayerActiveCache(name, nil, nil, role, GUID) -- (name, stealth, dead, role, GUID)
        return
    end
end

--> Is Unit Valid For Tracking
function NS.IsUnitValidForTracking(unit)
    if (not unit) or (not UnitIsPlayer(unit)) then -- input check
        return false
    end
    if string_find(GetUnitName(unit), "Unknown") then
        return false
    end
    if (UnitCanAttack("player", unit) or UnitIsEnemy("player", unit) or NS.Options.DEBUG) then -- attackable player check
        if not string_find(UnitGUID(unit), "Player") then -- player GUID check
            return false
        end
        return true -- VALID
    end
    return false
end

--> weizPVP: GetUnitName()
--: @returns "playerName-realmName" for all players, no matter the realm
function NS.GetUnitName(unit)
    if not unit then -- check for unit
        return
    end
    local name, realm = UnitName(unit, true) -- check for unit name
    if not name then
        return
    end
    if not realm then -- same realm
        return name .. "-" .. NS.PlayerRealm
    else -- different realm
        realm = gsub(realm, "[%s%-]", "")
        return name .. "-" .. realm
    end
end

--> UnitHealthCheck
function NS.UnitHealthCheck(unit)
    if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
        return -- no valid unit to process
    end
    local GUID = UnitGUID(unit) or nil
    if GUID and NS.PlayersOnBars[GUID] and NS.PlayerActiveCache[GUID] then -- only update if the player is shown on a bar
        local dead = UnitIsDeadOrGhost(unit)
        if dead ~= nil then
            NS.PlayerActiveCache[GUID].Dead = dead
        end
        local currentHealth = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
        if currentHealth ~= nil then
            NS.PlayerActiveCache[GUID].Health = currentHealth
        end
        if dead or currentHealth then
            NS.RefreshBarByGUID(GUID)
        end
    end
end
